@@ -2,7 +2,7 @@ package App::perlbrew;
 use strict;
 use warnings;
 use 5.008;
-our $VERSION = "0.45";
+our $VERSION = "0.46";
 
 use Config;
 use Capture::Tiny;
@@ -1003,6 +1003,36 @@ FAIL
     return;
 }
 
+sub do_install_program_from_url {
+    my ($self, $url, $program_name, $body_filter) = @_;
+
+    my $out = $self->root . "/bin/" . $program_name;
+
+    if (-f $out && !$self->{force}) {
+        require ExtUtils::MakeMaker;
+
+        my $ans = ExtUtils::MakeMaker::prompt("\n$out already exists, are you sure to override ? [y/N]", "N");
+
+        if ($ans !~ /^Y/i) {
+            print "\n$program_name installation skipped.\n\n" unless $self->{quiet};
+            return;
+        }
+    }
+
+    my $body = http_get($url) or die "\nERROR: Failed to retrieve $program_name executable.\n\n";
+
+    if ($body_filter && ref($body_filter) eq "CODE") {
+        $body = $body_filter->($body);
+    }
+
+    mkpath("@{[ $self->root ]}/bin") unless -d "@{[ $self->root ]}/bin";
+    open my $OUT, '>', $out or die "cannot open file($out): $!";
+    print $OUT $body;
+    close $OUT;
+    chmod 0755, $out;
+    print "\n$program_name is installed to\n\n    $out\n\n" unless $self->{quiet};
+}
+
 sub do_system {
   my ($self, @cmd) = @_;
   return ! system(@cmd);
@@ -1392,99 +1422,27 @@ sub run_command_symlink_executables {
     }
 }
 
-sub run_command_install_cpanm {
-    my ($self, $perl) = @_;
-    my $out = "@{[ $self->root ]}/bin/cpanm";
-
-    if (-f $out && !$self->{force}) {
-        require ExtUtils::MakeMaker;
-
-        my $ans = ExtUtils::MakeMaker::prompt("\n$out already exists, are you sure to override ? [y/N]", "N");
-
-        if ($ans !~ /^Y/i) {
-            print "\ncpanm installation skipped.\n\n"
-                unless $self->{quiet};
-            exit;
-        }
-    }
-
-    my $body = http_get('https://github.com/miyagawa/cpanminus/raw/master/cpanm');
-
-    unless ($body) {
-        die "\nERROR: Failed to retrieve cpanm executable.\n\n";
-    }
-
-    mkpath("@{[ $self->root ]}/bin") unless -d "@{[ $self->root ]}/bin";
-
-    open my $CPANM, '>', $out or die "cannot open file($out): $!";
-    print $CPANM $body;
-    close $CPANM;
-    chmod 0755, $out;
-
-    print "\ncpanm is installed to\n\n\t$out\n\n"
-        unless $self->{quiet};
-}
-
 sub run_command_install_patchperl {
     my ($self) = @_;
-    my $out = "@{[ $self->root ]}/bin/patchperl";
-
-    if (-f $out && !$self->{force}) {
-        require ExtUtils::MakeMaker;
-
-        my $ans = ExtUtils::MakeMaker::prompt("\n$out already exists, are you sure to override ? [y/N]", "N");
-
-        if ($ans !~ /^Y/i) {
-            print "\npatchperl installation skipped.\n\n"
-                unless $self->{quiet};
-            exit;
+    $self->do_install_program_from_url(
+        'https://raw.github.com/gugod/patchperl-packing/master/patchperl',
+        'patchperl',
+        sub {
+            my ($body) = @_;
+            $body =~ s/\A#!.+?\n/ $self->system_perl_shebang . "\n" /se;
+            return $body;
         }
-    }
+    );
+}
 
-    my $body = http_get('https://raw.github.com/gugod/patchperl-packing/master/patchperl');
-
-    unless ($body) {
-        die "\nERROR: Failed to retrieve patchperl executable.\n\n";
-    }
-
-    mkpath("@{[ $self->root ]}/bin") unless -d "@{[ $self->root ]}/bin";
-    open my $OUT, '>', $out or die "cannot open file($out): $!";
-
-    $body =~ s/\A#!.+?\n/ $self->system_perl_shebang . "\n" /se;
-    print $OUT $body;
-    close $OUT;
-    chmod 0755, $out;
-
-    print "\npatchperl is installed to\n\n\t$out\n\n"
-        unless $self->{quiet};
+sub run_command_install_cpanm {
+    my ($self) = @_;
+    $self->do_install_program_from_url('https://github.com/miyagawa/cpanminus/raw/master/cpanm' => 'cpanm');
 }
 
 sub run_command_install_ack {
     my ($self) = @_;
-    my $out = $self->root . "/bin/ack";
-
-    if (-f $out && !$self->{force}) {
-        require ExtUtils::MakeMaker;
-
-        my $ans = ExtUtils::MakeMaker::prompt("\n$out already exists, are you sure to override ? [y/N]", "N");
-
-        if ($ans !~ /^Y/i) {
-            print "\nack installation skipped.\n\n"
-            unless $self->{quiet};
-            return;
-        }
-    }
-
-    my $body = http_get('http://betterthangrep.com/ack-standalone');
-
-    unless ($body) {
-        die "\nERROR: Failed to retrieve ack executable.\n\n";
-    }
-
-    mkpath("@{[ $self->root ]}/bin") unless -d "@{[ $self->root ]}/bin";
-    open my $OUT, '>', $out or die "cannot open file($out): $!";
-
-    print "\nack is installed to\n\n\t$out\n\n" unless $self->{quiet};
+    $self->do_install_program_from_url('http://betterthangrep.com/ack-standalone' => 'ack');
 }
 
 sub run_command_self_upgrade {
@@ -1516,7 +1474,7 @@ sub run_command_self_upgrade {
         print "Your perlbrew is up-to-date.\n";
         return;
     }
-    system $TMP_PERLBREW, "install";
+    system $TMP_PERLBREW, "self-install";
     unlink $TMP_PERLBREW;
 }
 
@@ -1561,7 +1519,17 @@ sub run_command_exec {
     my @exec_with = map { ($_, @{$_->{libs}}) } $self->installed_perls;
 
     if ($opts{with}) {
-        @exec_with = grep { $_->{name} eq $opts{with} } @exec_with;
+        my $d = ($opts{with} =~ / /) ? qr( +) : qr(,+);
+        my %x = map { $_ => 1 } grep { $_ } map {
+            my ($p,$l) = $self->resolve_installation_name($_);
+            $p .= "\@$l" if $l;
+            $p;
+        } split $d, $opts{with};
+        @exec_with = grep { $x{ $_->{name} } } @exec_with;
+    }
+
+    if (0 == @exec_with) {
+        print "No perl installation found.\n" unless $self->{quiet};
     }
 
     for my $i ( @exec_with ) {
@@ -1936,7 +1904,7 @@ __perlbrew_set_path () {
     fi
 
     export PATH="$PERLBREW_PATH:$PATH_WITHOUT_PERLBREW"
-    export MANPATH_WITHOUT_PERLBREW="$(perl -e 'print join ":", grep { index($_, $ENV{PERLBREW_ROOT}) } split/:/,$ENV{MANPATH};')"
+    export MANPATH_WITHOUT_PERLBREW="$(perl -e 'print join ":", grep { index($_, $ENV{PERLBREW_ROOT}) } split/:/,qx(manpath);')"
     if [ -n "$PERLBREW_MANPATH" ]; then
         export MANPATH="$PERLBREW_MANPATH:$MANPATH_WITHOUT_PERLBREW"
     else
@@ -2123,7 +2091,7 @@ endif
 setenv PATH_WITHOUT_PERLBREW `perl -e 'print join ":", grep { index($_, $ENV{PERLBREW_ROOT}) } split/:/,$ENV{PATH};'`
 setenv PATH ${PERLBREW_PATH}:${PATH_WITHOUT_PERLBREW}
 
-setenv MANPATH_WITHOUT_PERLBREW `perl -e 'print join ":", grep { index($_, $ENV{PERLBREW_ROOT}) } split/:/,$ENV{MANPATH};'`
+setenv MANPATH_WITHOUT_PERLBREW `perl -e 'print join ":", grep { index($_, $ENV{PERLBREW_ROOT}) } split/:/,qx(manpath);'`
 if ( $?PERLBREW_MANPATH == 1 ) then
     setenv MANPATH ${PERLBREW_MANPATH}:${MANPATH_WITHOUT_PERLBREW}
 else
